/*
 * Copyright (c) 2018 ARM Limited
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef __SMM_MPS3_H__
#define __SMM_MPS3_H__

#include "cmsis.h"   /* device specific header file */

/* FPGAIO register map structure */
struct arm_mps3_fpgaio_t {
    volatile uint32_t LED;           /* Offset: 0x000 (R/W) LED connections
                                      *         [31:2] : Reserved
                                      *         [1:0]  : LEDs */
    volatile uint32_t RESERVED1[1];
    volatile uint32_t BUTTON;        /* Offset: 0x008 (R/W) Buttons
                                      *         [31:2] : Reserved
                                      *         [1:0]  : Buttons */
    volatile uint32_t RESERVED2[1];
    volatile uint32_t CLK1HZ;        /* Offset: 0x010 (R/W) 1Hz up counter */
    volatile uint32_t CLK100HZ;      /* Offset: 0x014 (R/W) 100Hz up counter */
    volatile uint32_t COUNTER;       /* Offset: 0x018 (R/W) Cycle Up Counter
                                      *                     Increments when
                                      *                     32bit prescale
                                      *                     counter reach
                                      *                     zero */
    volatile uint32_t PRESCALE;      /* Offset: 0x020 (R/W) Prescaler
                                      *                     Bit[31:0] : reload
                                      *                     value for prescale
                                      *                     counter */
    volatile uint32_t PSCNTR;        /* Offset: 0x024 (R/W) 32bit Prescale
                                      *                     counter. Current
                                      *                     value of the
                                      *                     prescaler counter.
                                      *
                                      * The Cycle Up Counter increment when the
                                      * prescale down counter reach 0.
                                      * The prescaler counter is reloaded with
                                      * PRESCALE after reaching 0. */
    volatile uint32_t RESERVED3[1];
    volatile uint32_t SWITCHES;      /* Offset: 0x028 (R/W)  Switches
                                                [31:8] : Reserved
                                                [7:0] : Switches. */
    volatile uint32_t RESERVED4[8];
    volatile uint32_t MISC;          /* Offset: 0x04C (R/W) Misc control
                                      *         [31:3] : Reserved
                                      *         [2] : SHIELD_1_SPI_nCS
                                      *         [1] : SHIELD_0_SPI_nCS
                                      *         [0] : ADC_SPI_nCS */
};

/* SCC register map structure */
struct arm_mps3_scc_t {
    volatile uint32_t CFG_REG0;    /* Offset: 0x000 (R/W) Remaps block RAM to
                                    *                     ZBT
                                    *         [31:1] : Reserved
                                    *         [0] 1  : REMAP BlockRam to ZBT */
    volatile uint32_t CFG_REG1;    /* Offset: 0x004 (R/W) DATA */
    volatile uint32_t CFG_REG2;    /* Offset: 0x008 (R/W)
                                    *         [31:1] : Reserved
                                    *         [0] 1  : QSPI Select signal */
    volatile uint32_t CFG_REG3;    /* Offset: 0x00C (R/W) RESERVED */
    volatile uint32_t CFG_REG4;    /* Offset: 0x010 (R)
                                    *         [31:4] : Reserved
                                    *         [3:0]  : Board Revision */
    volatile uint32_t CFG_REG5;    /* Offset: 0x014 (R/W) ACLK Frequency in 
                                                       HZ */

    volatile uint32_t RESERVED0[32];


    volatile uint32_t SYS_CFGDATA_RTN; /* Offset: 0x0A0 (R/W) User data register
                                        *         [31:0] : Data */
    volatile uint32_t SYS_CFGDATA_OUT; /* Offset: 0x0A4 (R/W)  User data
                                        *                      register
                                        *         [31:0] : Data */
    volatile uint32_t RESERVED1[978];
    volatile uint32_t SCC_AID;         /* Offset: 0xFF8 (R/ ) SCC AID Register
                                        *         [31:24] : FPGA build number
                                        *         [23:20] : V2MMPS3 target
                                        *                   board revision
                                        *                   (A = 0, B = 1)
                                        *         [19:8] : Reserved
                                        *         [7:0]   : number of SCC
                                        *                   configuration
                                        *                   register */
    volatile uint32_t SCC_ID;          /* Offset: 0xFFC (R/ ) Contains
                                        *                     information about
                                        *                     the FPGA image
                                        *         [31:24] : Implementer ID:
                                        *                   0x41 = ARM
                                        *         [23:20] : Application note
                                        *                   IP variant number
                                        *         [19:16] : IP Architecture:
                                        *                   0x4 =AHB
                                        *         [11:4]  : Primary part number:
                                        *                   524 = AN524
                                        *         [3:0]   : Application note IP
                                        *                   revision number */
};

/* Peripheral memory map */
#define MPS3_FPGAIO_BASE_NS  0x41302000  /* FPGAIO Base Address */
#define MPS3_SCC_BASE_NS     0x41300000  /* SCC Base Address */

/* Secure Peripheral memory map */
#define MPS3_FPGAIO_BASE_S   0x51302000  /* FPGAIO Base Address */
#define MPS3_SCC_BASE_S      0x51300000  /* SCC Base Address */

/* Peripheral declaration */
#define MPS3_FPGAIO      ((struct arm_mps3_fpgaio_t*) MPS3_FPGAIO_BASE_NS)
#define MPS3_SCC         ((struct arm_mps3_scc_t*) MPS3_SCC_BASE_NS)

/* Secure Peripheral declaration */
#define SEC_MPS3_FPGAIO  ((struct arm_mps3_fpgaio_t*) MPS3_FPGAIO_BASE_S)
#define SEC_MPS3_SCC     ((struct arm_mps3_scc_t*) MPS3_SCC_BASE_S)

#endif /* __SMM_MPS3_H__ */
