/*
 * Copyright (c) 2018 ARM Limited
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <stdint.h>
#include "mps3_time.h"
#include "arm_mps3_io_drv.h"
#include "platform_retarget_dev.h"

/*
 * Semihosting is a mechanism that enables code running on an ARM target
 * to communicate and use the Input/Output facilities of a host computer
 * that is running a debugger.
 * There is an issue where if you use armclang at -O0 optimisation with
 * no parameters specified in the main function, the initialisation code
 * contains a breakpoint for semihosting by default. This will stop the
 * code from running before main is reached.
 * Semihosting can be disabled by defining __ARM_use_no_argv symbol
 * (or using higher optimization level).
 */
#if defined (__ARMCC_VERSION) && (__ARMCC_VERSION >= 6010050)
__asm("  .global __ARM_use_no_argv\n");
#endif

#define LED_ON    1
#define LED_OFF   0

/**
 * \brief Run LED light on available LEDs by switching 1 LED on
 *        one after one in one direction, then backwards and
 *        start again.
 *
 * \param[in] led_port_bit_length Bit length of the LED port.
 * \param[in] delay_ms Delay between LED state changes in ms.
 */
void led_runner(unsigned int led_port_bit_length, unsigned int delay_ms) __attribute__((noreturn));

int main(void)
{
    const uint32_t led_num = 8;
    const uint32_t delay_ms = 200;

    led_runner(led_num, delay_ms);

    /* Never reach here */
    return 0;
}

void led_runner(unsigned int led_num, unsigned int delay_ms)
{
    uint32_t direction = 0;
    uint8_t led = 0;

    /* Turn off all LED firstly */
    arm_mps3_io_write_leds(&ARM_MPS3_IO_FPGAIO_DEV_S,
                            ARM_MPS3_IO_ACCESS_PORT,
                            0,
                            LED_OFF);

    for (;;) {
        if (!direction) {
            if (led < led_num - 1) {
                led++;
            } else {
                direction = 1;
                led--;
            }
        } else {
            if (led > 0) {
                led--;
            } else {
                direction = 0;
                led++;
            }
        }
        arm_mps3_io_write_leds(&ARM_MPS3_IO_FPGAIO_DEV_S,
                               ARM_MPS3_IO_ACCESS_PIN,
                               led,
                               LED_ON);

        mps3_sleepms(delay_ms);

        arm_mps3_io_write_leds(&ARM_MPS3_IO_FPGAIO_DEV_S,
                               ARM_MPS3_IO_ACCESS_PIN,
                               led,
                               LED_OFF);
    }
}
